drop table bottom cascade constraints;
drop table middle cascade constraints;
drop table top cascade constraints;
drop table a cascade constraints;
drop table b cascade constraints;
drop table c cascade constraints;
drop synonym btm;

create table top (
  id 		integer 	not null,
  hoeveelheid 	number 		not null,
  timestamp 	date 		null
)
/

create table middle (
  id 		integer 	not null,
  hoeveelheid 	number 		null,
  timestamp 	date 		null,
  top_id	integer		null,
  top_hoeveelheid integer       null
)
/

create table bottom (
  id 		integer 	not null,
  hoeveelheid 	number 		null,
  timestamp 	date 		null,
  middle_id	integer		null,
  number10_2    number(10,2)    null,
  number13      number(13)      null,
  nr            number          null,
  vc100         varchar2(100)   null,
  c100          char(100)       null,
  lng           long            null,
  raw100        raw(100)        null 
)
/

ALTER TABLE BOTTOM ADD (
      CONSTRAINT BOTTOM_PK
      PRIMARY KEY (ID)
USING INDEX 
PCTFREE  10
)
/

ALTER TABLE MIDDLE ADD (
      CONSTRAINT MIDDLE_PK
      PRIMARY KEY (ID)
USING INDEX 
PCTFREE  10
)
/

ALTER TABLE TOP ADD (
      CONSTRAINT TOP_PK
      PRIMARY KEY (ID,hoeveelheid)
USING INDEX 
PCTFREE  10
)
/

ALTER TABLE BOTTOM ADD (
      CONSTRAINT BOTTOM_MIDDLE_FK
      FOREIGN KEY (MIDDLE_ID)
      REFERENCES  MIDDLE (
                   ID)
)
/

ALTER TABLE MIDDLE ADD (
      CONSTRAINT MIDDLE_TOP_FK
      FOREIGN KEY (TOP_ID,top_hoeveelheid)
      REFERENCES  TOP (
                   ID,hoeveelheid)
)
/

CREATE INDEX BOTTOM_MIDDLE_FK_I ON BOTTOM
(
      middle_id )
PCTFREE  10
;


CREATE INDEX MIDDLE_TOP_FK_I ON MIDDLE
(
      top_id,top_hoeveelheid )
PCTFREE  10
;

COMMENT ON TABLE top IS 'Top Table';
COMMENT ON TABLE middle IS 'Middle Table';
COMMENT ON TABLE bottom IS 'Bottom Table';
COMMENT ON COLUMN top.id IS 'Top Table ID';
COMMENT ON COLUMN middle.id IS 'Middle Table ID';
COMMENT ON COLUMN bottom.id IS 'Bottom Table ID';

create or replace trigger bottom_trigger
before insert on bottom
for each row
begin
  :new.hoeveelheid:=:new.hoeveelheid*2;
end;
/

create or replace trigger bottom_trigger2
after update of hoeveelheid,id or insert on bottom
referencing new as xxx old as yyy
for each row
when (yyy.id=1)
declare
  hoeveelheid number;
begin
  hoeveelheid:=:xxx.hoeveelheid*2;
end;
/

begin
  for x in 1..10 loop
    insert into top (id,hoeveelheid,timestamp) values (x,x,sysdate);
    insert into middle (id,hoeveelheid,timestamp,top_id) values (x,x,sysdate,x);
    insert into bottom (id,hoeveelheid,timestamp,middle_id) values (x,x,sysdate,x);
  end loop;
  commit;
end;
/

create table a (id number not null,c_id number)
/

create table b (id number not null,a_id number)
/

create table c (id number not null,b_id number)
/

ALTER TABLE a ADD (
      CONSTRAINT a_PK
      PRIMARY KEY (ID)
USING INDEX 
PCTFREE  10
)
/

ALTER TABLE b ADD (
      CONSTRAINT b_PK
      PRIMARY KEY (ID)
USING INDEX 
PCTFREE  10
)
/

ALTER TABLE c ADD (
      CONSTRAINT c_PK
      PRIMARY KEY (ID)
USING INDEX 
PCTFREE  10
)
/

ALTER TABLE a add (
      CONSTRAINT a_FK
      FOREIGN KEY (c_ID)
      REFERENCES  c (
                   ID)
)
/

ALTER TABLE b add (
      CONSTRAINT b_FK
      FOREIGN KEY (a_ID)
      REFERENCES  a (
                   ID)
)
/

ALTER TABLE c add (
      CONSTRAINT c_FK
      FOREIGN KEY (b_ID)
      REFERENCES  b (
                   ID)
)
/


insert into a values (1,null);
insert into b values (2,1);
insert into c values (3,2);
insert into a values (4,3);
insert into c values (2,null);
insert into a values (5,2);
update a set c_id=3 where id=1;
commit;

grant select on bottom to system;
grant update (id,hoeveelheid) on bottom to system;
create synonym btm for bottom;
